import argparse
from .config import get_config
from .handlers import (
    remove_component,
    pull_components,
    add_component,
    get_component,
    tree_view,
    init_app,
    push_app,
)

parser = argparse.ArgumentParser('swp')
parser.add_argument('-c', metavar='PATH', type=str, help="Configuration path", default="swap.yaml")

# SUBPARSER CONFIG
subparser = parser.add_subparsers(
    dest='action', title='action', description='SWAP actions', required=True)

# INIT
init = subparser.add_parser('init', help='initialize a new project')
init.add_argument('--remote', '-r', help='git repository url')
init.add_argument('--folder', '-f', help='subfolder path')
init.set_defaults(handler=init_app, require_config=False)

# TREE
tree = subparser.add_parser('tree', help='show tree view of remote')
tree.set_defaults(handler=tree_view, require_config=True)

# PUSH
push = subparser.add_parser('push', help='push components to remote')
push.add_argument('MESSAGE', nargs='?', help='commit message')
push.set_defaults(handler=push_app, require_config=True)

# PULL
pull = subparser.add_parser('pull', help='pull components from remote')
pull.set_defaults(handler=pull_components, require_config=True)

# ADD
add = subparser.add_parser('add', help='add component to the project')
add.add_argument('PATH', nargs='+', help='path of the component')
add.set_defaults(handler=add_component, require_config=True)

# GET
get = subparser.add_parser('get', help='get component locally from remote')
get.add_argument('NAME', help='component name to get')
get.add_argument('DEST', help='path where component should be created')
get.set_defaults(handler=get_component, require_config=True)

# REMOVE
remove = subparser.add_parser('rm', help='remove component from the project')
remove.add_argument('PATH', nargs='+', help='path of the component')
remove.set_defaults(handler=remove_component, require_config=True)

def main():
    # Parse arguments
    options = parser.parse_args()

    # Load local config
    try:
        options.config = get_config(options.c)
    except:
        options.config = None

    # Check if local config is required
    if options.require_config and not options.config:
        exit('You should have init a project before running this command')

    # Execute the command
    if options.handler:
        options.handler(options)



from .git import git_clone, git_pull, git_push, git_add_commit
from .questions import get_remote_url, get_sub_folder
from .config import save_config
from .tree import list_files

from distutils.dir_util import copy_tree
from shutil import copy2
from os import path


def get_work_dir(config):
    folder = config['remote'].split('/')[-1].split('.')[0]
    return path.join('/tmp', folder)


def check_path(item):
    if path.isfile(item):
        return True
    elif path.isdir(item):
        return True
    exit('You must choose a file or a directory')


def init_app(options):
    if path.exists(options.c):
        exit('Cannot override existing configuration')

    config = {
        'version': 1,
        'remote': get_remote_url(options),
        'remote-directory': get_sub_folder(options),
        'components': {},
    }

    save_config(config, options.c)
    git_clone(config['remote'], get_work_dir(config))


def tree_view(options):
    work_dir = get_work_dir(options.config)
    subdest = path.join(work_dir, options.config['remote-directory'])
    list_files(subdest)


def push_app(options):
    work_dir = get_work_dir(options.config)
    subdest = path.join(work_dir, options.config['remote-directory'])

    if not path.exists(work_dir):
        git_clone(options.config['remote'], work_dir)
    else:
        git_pull(work_dir)

    for name, item_path in options.config['components'].items():
        if path.isdir(item_path):
            copy_tree(item_path, path.join(subdest, name))
        else:
            copy2(item_path, path.join(subdest, name))

    git_add_commit(work_dir, options.MESSAGE or None)
    git_push(work_dir)


def pull_components(options):
    # TODO: Detect unvalidated changes in git and ask if it's ok
    work_dir = get_work_dir(options.config)
    subdest = path.join(work_dir, options.config['remote-directory'])

    if not path.exists(work_dir):
        git_clone(options.config['remote'], work_dir)
    else:
        git_pull(work_dir)

    for name, item_path in options.config['components'].items():
        if path.isdir(path.join(subdest, name)):
            copy_tree(path.join(subdest, name), item_path)
        else:
            copy2(path.join(subdest, name), item_path)


def add_component(options):
    for item in options.PATH:
        check_path(item)
        name = path.basename(item)
        if name not in options.config['components']:
            options.config['components'][name] = path.normpath(item)
        else:
            print('Cannot add 2 component with the same name')
    save_config(options.config, options.c)


def get_component(options):
    work_dir = get_work_dir(options.config)
    subdest = path.join(work_dir, options.config['remote-directory'])
    name = options.NAME
    dest = options.DEST

    if not path.exists(work_dir):
        git_clone(options.config['remote'], work_dir)
    else:
        git_pull(work_dir)

    if path.exists(dest):
        exit(f'This path already exists {dest}')
    if not path.exists(path.join(subdest, name)):
        exit(f'No components found for this name {name}')
    if path.isdir(path.join(subdest, name)):
        copy_tree(path.join(subdest, name), dest)
    else:
        copy2(path.join(subdest, name), dest)


def remove_component(options):
    for item in options.PATH:
        check_path(item)
        if path.basename(item) in options.config['components']:
            options.config['components'].pop(path.basename(item))
        else:
            print(f'Cannot find the component {path.basename(item)}')
    save_config(options.config, options.c)



import subprocess
import os

def list_files(startpath):
    if os.path.exists('/bin/tree'):
        subprocess.run(f'/bin/tree {startpath}', shell=True)
        return

    for root, _, files in os.walk(startpath):
        level = root.replace(startpath, '').count(os.sep)
        indent = ' ' * 2 * (level)
        if '.git' in root:
            continue
        print('{}{}/'.format(indent, os.path.basename(root)))
        subindent = ' ' * 2 * (level + 1)
        for f in files:
            print('{}{}'.format(subindent, f))



from PyInquirer import prompt


def get_sub_folder(options):
    if options.folder:
        return options.folder
    question = {
        'type': 'input',
        'name': 'folder',
        'default': './',
        'message': 'Sub folder containing all the components',
    }
    return prompt(question)['folder']


def get_remote_url(options):
    if options.remote:
        return options.remote
    question = {
        'type': 'input',
        'name': 'remote',
        'message': 'Git remote url for saving these components',
    }
    return prompt(question)['remote']
    


from datetime import datetime
import subprocess

OPTIONS = {
    'shell': True
}

def git_clone(remote, dest):
    subprocess.check_output(f'git clone {remote} {dest}', **OPTIONS)


def git_pull(dest):
    subprocess.check_output(f'git -C {dest} pull', **OPTIONS)


def git_push(dest):
    subprocess.check_output(f'git -C {dest} push origin master', **OPTIONS)


def git_add_commit(dest, message=None):
    subprocess.check_output(f'git -C {dest} add --all', **OPTIONS)
    message = message or f'[UPDATE] {datetime.now()}'
    subprocess.check_output(f'git -C {dest} commit -m "{message}"', **OPTIONS)



from re import compile
from os import path
import yaml

variables = compile(r'\s(\$[A-Z_]+)\s')


def get_config(config_path):
    if not path.exists(config_path):
        return None
    with open(config_path) as fp:
        config = yaml.load(fp, yaml.SafeLoader)
    return config


def save_config(config, config_path):
    with open(config_path, 'w+') as fp:
        yaml.dump(config, fp, default_flow_style=False, indent=2)



from setuptools import setup, find_namespace_packages

setup(
    name='swp',
    version='0.1.3',
    description='Simple components sharing tool',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/mathix420/swap',
    author='Arnaud Gissinger',
    author_email='agissing@student.42.fr',
    license='MIT',
    python_requires='>=3.6',
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    install_requires=open('requirements.txt').read().split('\n'),
    packages=find_namespace_packages(include=["swap", "swap.*"]),
    entry_points={'console_scripts': ['swp=swap.__main__:main']},
)
